<?php
c_include('extApp');

class GoogleExtApp extends ExtApp {
	var $goo_client_obj;
	var $goo_oauth_obj;
	var $goo_youtube_obj;
	
	function __construct($aplicacion){
		parent::__construct($aplicacion);
	}
	
	function includes(){
		c_include('extension','app','usuario');
		
		set_include_path(get_include_path() . PATH_SEPARATOR . CONF_REALPATH_APP . 'apps/app_google/tools/src');
		
		if (!class_exists('Google_Client')){
			require(CONF_REALPATH_APP."apps/app_google/tools/src/Google/Client.php");
		}
		
		if (!class_exists('Google_Service_Oauth2')){
			require(CONF_REALPATH_APP."apps/app_google/tools/src/Google/Service/Oauth2.php");
		}
		
		if (!class_exists('Google_Service_Analytics')){
			require(CONF_REALPATH_APP."apps/app_google/tools/src/Google/Service/Analytics.php");
		}
		
		if (!class_exists('Google_Http_Request')){
			require(CONF_REALPATH_APP."apps/app_google/tools/src/Google/Http/Request.php");
		}
		
		if (!class_exists('Google_Service_YouTube')){
			require(CONF_REALPATH_APP."apps/app_google/tools/src/Google/Service/YouTube.php");
		}
	}
	
	function registerWidgets(){
		$widgets = array('google_box'=>array('name'=>__('Caja de fans de Google+'), 'folder'=>CONF_REALPATH_APP . 'apps/app_google/tools/widgets/google_box/'));
		
		return $widgets;
	}
	
	function addScripts(){
		if (($this->parametros['gestionar_estadisticas'] == '1' && !empty($this->parametros['codigo_seguimiento']) && !$GLOBALS['apps_added_scripts'][$this->carpeta]) || ($this->parametros['gestionar_mapas'] == '1' && !empty($this->parametros['apiKey']))){
			if ($this->parametros['gestionar_estadisticas'] == '1' && !empty($this->parametros['codigo_seguimiento']) && !$GLOBALS['apps_added_scripts'][$this->carpeta]){
				$GLOBALS['AppDoc']->addScriptDeclaration((string) $this->parametros['codigo_seguimiento']);
			}
			
			if ($this->parametros['gestionar_mapas'] == '1' && !empty($this->parametros['apiKey'])){
				global $AssetBundles;
		
				if ($AssetBundles instanceof AssetsHelper){
					$googlemaps_bundle = new AssetBundle(array('id'=>'googlemaps', 'nombre'=>'Google Maps'));
					
					$googlemaps_bundle->register_js_top('googlemaps-api', 'http://maps.googleapis.com/maps/api/js?key='.$this->parametros['apiKey'].'&amp;libraries=places', array('externo'=>true));
					$googlemaps_bundle->register_js('gmaps', CONF_URL_APP.CONF_DIR_THEMES.'/sistema/assets/js/gmaps.js');
					$googlemaps_bundle->register_js('mideas-gmaps', CONF_URL_APP.CONF_DIR_THEMES.'/sistema/js/mideas/mideas.gmaps.js');
					
					$AssetBundles->register_bundle($googlemaps_bundle);
				}
			}
		
			$GLOBALS['apps_added_scripts'][$this->carpeta] = true;
		}
	}
	
	function addMetas($metas=array()){
		if (!is_array($metas) || sizeof($metas) == 0){
			return false;
		}
		
		$metas_parseados = array("name"=>$metas['titulo'], "image"=>$metas['imagen'], "description"=>$metas['descripcion']);
		
		$metas_finales = array();
		foreach ($metas_parseados as $name_meta=>$content_meta){
			if (!empty($content_meta) && !array_key_exists($name_meta, $GLOBALS['AppDoc']->_metaTags['itemprop'])){
				$metas_finales[$name_meta] = $content_meta;
				$GLOBALS['AppDoc']->_metaTags['itemprop'][$name_meta] = $content_meta;
			}
		}
		
		return $metas_finales;
		
	}
	
	/* FUNCIONES DE USUARIOS */
	
	function authenticateApp($param='gestionar_usuarios', $redirect_uri, $scopes=array(/*"https://www.googleapis.com/auth/youtube","https://www.googleapis.com/auth/contacts.readonly","https://www.googleapis.com/auth/analytics.readonly",*/"https://www.googleapis.com/auth/userinfo.email", "https://www.googleapis.com/auth/userinfo.profile")){
		if (empty($param))
			$param = 'gestionar_usuarios';
		
		if ($this->isAuthenticatedApp() && is_object($this->goo_client_obj)) {
			if ($this->goo_client_obj->getAccessToken()){
				return true;
			} else {
				$authUrl = $this->goo_client_obj->createAuthUrl();
 				Helpers::redirect($authUrl);
				exit;
			}
		} else {
			$clientId = $this->parametros['clientId'];
			$secret = $this->parametros['clientSecret'];
			$apiKey = $this->parametros['apiKey'];
			
			$activo = $this->activo == '1' ? true : false;
			
			if ($activo)
				$activo = $this->parametros[$param] == '1' ? true : false;
			
			if (empty($clientId) || empty($secret) || !$activo){
				return __('No se puede usar la aplicación de Google');
			}
			$this->goo_client_obj = new Google_Client();
			$this->goo_client_obj->setScopes($scopes);
			$this->goo_client_obj->setApplicationName($this->nombre);
			$this->goo_client_obj->setClientId($clientId);
			$this->goo_client_obj->setClientSecret($secret);
			$this->goo_client_obj->setDeveloperKey($apiKey);
			$this->goo_client_obj->setAccessType("online");
                        
			if (empty($redirect_uri)){
				//$redirect_uri = $this->getApiUrl('autorizar', 'system-msg') . '/';
           		$redirect_uri = Helpers::currentURL().'/';
			} elseif (substr($redirect_uri, -4) != '.php' && substr($redirect_uri, -1) != '/') {
				$redirect_uri = $redirect_uri . '/';
			}/* else {
				$redirect_uri = CONF_URL_APP . $redirect_uri;
			}*/
			if (substr($redirect_uri, 0, 7) != 'http://' && substr($redirect_uri, 0, 8) != 'https://'){
				$redirect_uri = CONF_URL_APP . $redirect_uri;
			}
			$this->goo_client_obj->setRedirectUri($redirect_uri);
			
			$this->goo_oauth_obj = new Google_Service_Oauth2($this->goo_client_obj);
			
			$token_goo = Helpers::getSesion()->get('apps.'.$this->nombre.'.token');
			
			if (isset($_GET['code']) && empty($token_goo)) {
			  $this->goo_client_obj->authenticate($_GET['code']);
			  
			  Helpers::getSesion()->set('apps.'.$this->nombre.'.token', $this->goo_client_obj->getAccessToken());
			  Helpers::redirect(Helpers::currentUrl());
			  exit;
			}
			
			$token_goo = Helpers::getSesion()->get('apps.'.$this->nombre.'.token');
			
			if (!empty($token_goo)) {
			 $this->goo_client_obj->setAccessToken($token_goo);
			}
			
			if ($this->goo_client_obj->getAccessToken()){
				Helpers::getSesion()->set('apps.'.$this->nombre.'.autorizado', true);
				return true;
			} else {
				$authUrl = $this->goo_client_obj->createAuthUrl();
 				Helpers::redirect($authUrl);
				exit;
			}
		}
	}
	
	function unauthenticateApp(){
		if ($this->isAuthenticatedApp()){
			$this->authenticateApp();
			Helpers::getSesion()->clear('apps.'.$this->nombre.'.autorizado');
			Helpers::getSesion()->clear('apps.'.$this->nombre.'.token');
			Helpers::getSesion()->clear('apps.'.$this->nombre.'.google_data');
			$this->goo_client_obj->revokeToken();
			return true;
		} else {
			return false;
		}
	}
	
	function isAuthenticatedApp(){
		return Helpers::getSesion()->get('apps.'.$this->nombre.'.autorizado');
	}
	
	function usrLogin(){
		$auth_usr = $this->authenticateApp('gestionar_usuarios', $this->getApiUrl('login', 'system-msg') . '/');
		
		if($auth_usr === true) {
			$user = $this->goo_oauth_obj->userinfo->get();
			$user['username'] = 'umd'.time();
			
			/*Se corrigió el error de cabecera: SESSION estaba debajo de print_r y no se ejecutaba antes del llamado*/
			Helpers::getSesion()->set('apps.'.$this->nombre.'.google_data', $user);
			
			$imagen_usuario = $user["picture"];
			
			if (!empty($imagen_usuario)){
				$rnd_fld = rand();
				$ext = explode('.',$imagen_usuario);
				$ext = $ext[count($ext)-1];
				if (empty($ext))
					$ext = 'jpg';
				$img_name = md5($imagen_usuario).'.'.$ext;
				if (!is_dir(CONF_REALPATH_APP.'uploads/imagenes/usuarios/'.$rnd_fld)){
					Helpers::ftpCrearCarpeta('uploads/imagenes/usuarios/'.$rnd_fld);
				}
				if (!is_file(CONF_REALPATH_APP.'uploads/imagenes/usuarios/'.$rnd_fld.'/'.$img_name)){
					$ch = curl_init($imagen_usuario);
					$fp = fopen(CONF_REALPATH_APP.'uploads/imagenes/usuarios/'.$rnd_fld.'/'.$img_name, "w");
					
					curl_setopt($ch, CURLOPT_FILE, $fp);
					curl_setopt($ch, CURLOPT_HEADER, 0);
					
					curl_exec($ch);
					curl_close($ch);
					fclose($fp);
					chmod(CONF_REALPATH_APP.'uploads/imagenes/usuarios/'.$rnd_fld.'/'.$img_name, 0777);
				}
				$imagen_usuario = 'uploads/imagenes/usuarios/'.$rnd_fld.'/'.$img_name;
			}
			# We have an active session, let's check if we have already registered the user
			$existe_user_fb = new Usuario();
			
			$existe_user_fb = $existe_user_fb->Listar('google_uid="'.$user['id'].'" AND estado=1 AND email="'.$user['email'].'"');
			
			if (is_array($existe_user_fb) && count($existe_user_fb) == 1){
				$existe_user_fb = $existe_user_fb[0];
				$id_usuario = $existe_user_fb->id;
				
				if (empty($existe_user_fb->imagen) && !empty($imagen_usuario)){
					$existe_user_fb->setimagen($imagen_usuario);
				}
				Helpers::getSesion()->set('usuario.ultimo_acceso', $existe_user_fb->fecha_ultimo_acceso);
				$existe_user_fb->setfecha_ultimo_acceso(strftime(SistemaIdioma::formatoFecha('sistema')));
				$existe_user_fb->setcontrasena('');
				$existe_user_fb->prepararModificar();
				$existe_user_fb->Modificar();
			} else {
				$usuario_mideas = new Usuario();
				$usuario_mideas = $usuario_mideas->Listar('estado=1 AND email="'.$user['email'].'"');
				if (is_array($usuario_mideas) && count($usuario_mideas) == 1){
					$usuario_mideas = $usuario_mideas[0];
					$usuario_mideas->setgoogle_uid($user['id']);
					if (empty($usuario_mideas->imagen) && !empty($imagen_usuario))
						$usuario_mideas->setimagen($imagen_usuario);
						
					Helpers::getSesion()->set('usuario.ultimo_acceso', $usuario_mideas->fecha_ultimo_acceso);
					$usuario_mideas->setfecha_ultimo_acceso(strftime(SistemaIdioma::formatoFecha('sistema')));
					$usuario_mideas->setcontrasena('');
					$usuario_mideas->prepararModificar();
					$usuario_mideas->Modificar();
					
					$id_usuario = $usuario_mideas->id;
				} else {
                                        $cat_default = $GLOBALS['opciones']['ext_usuarios']['general']['categoria_defecto'];
					$nombre_usr = $user['given_name'];
					if (!empty($user['family_name'])){
						$nombre_usr .= ' '.$user['family_name'];
					}
                                        
					$nuevo_usuario = new Usuario();
					$nuevo_usuario->setgoogle_uid($user['id']);
					$nuevo_usuario->setnombre($nombre_usr);
					$nuevo_usuario->setusername($user['username']);
					$nuevo_usuario->setemail($user['email']);
                                        $nuevo_usuario->setcategorias(json_encode(array($cat_default)));
					if (!empty($imagen_usuario))
						$nuevo_usuario->setimagen($imagen_usuario);
					$nuevo_usuario->setestado(1);
					$nuevo_usuario->setnivel_acceso(1);
					Helpers::getSesion()->set('usuario.ultimo_acceso', strftime(SistemaIdioma::formatoFecha('sistema')));
					$nuevo_usuario->setfecha_ultimo_acceso(strftime(SistemaIdioma::formatoFecha('sistema')));
					$id_usuario = $nuevo_usuario->Guardar();
					
					//$id_usuario = mysqli_insert_id();
				}
			}
			
			$usuario_loged = new Usuario();
			$usuario_loged->setid($id_usuario);
			$usuario_loged = $usuario_loged->BuscarId();
			if (!$usuario_loged){
                                return __("No se ha encontrado el usuario. Error crítico!");
				/*App::setMsg(__("No se ha encontrado el usuario. Error crítico!"));
				if ($this->parametros['usuarios_boton_accion'] == 'popup'){
					echo '<script>opener.location.reload(true);window.close();</script>'; 
				} else {
					Helpers::redirect($_SESSION[CLAVE_SECRETA]['site']['lastUrl']);
				}
				exit;*/
			}
			
			$AppUsuario = $usuario_loged->username;
			
			Helpers::getSesion()->set('usuario.AppIdUsuario', $id_usuario);
			Helpers::getSesion()->set('usuario.AppUsuario', $AppUsuario);
			Helpers::getSesion()->set('usuario.nivel_acceso', $usuario_loged->nivel_acceso);
			Helpers::getSesion()->set('usuario.google', true);
			
			if ($save_in_cookie)
				self::setLoginCookie($id_usuario);
			
			return true;
			
			// The access token may have been updated lazily.
			Helpers::getSesion()->set('apps.'.$this->nombre.'.token', $client->getAccessToken());
		} else {
                        return $auth_usr;
			/*App::setMsg($auth_usr, 'error');
			if ($this->parametros['usuarios_boton_accion'] == 'popup'){
				echo '<script>opener.location.reload(true);window.close();</script>';
			} else {
				Helpers::redirect($_SESSION[CLAVE_SECRETA]['site']['lastUrl']);
			}
			exit;*/
		}
		
		return false;
	}
	
	function usrLogout(){
		if ($this->usrIsLoged()){
			/*$this->authenticateApp();
			@session_start();
			unset($_SESSION[CLAVE_SECRETA]['apps'][$this->nombre]['autorizado']);
			unset($_SESSION[CLAVE_SECRETA]['apps'][$this->nombre]['token']);
			unset($_SESSION[CLAVE_SECRETA]['apps'][$this->nombre]['google_data']); //Google session data unset
			$this->goo_client_obj->revokeToken();*/
			return true;
		} else {
			return false;
		}
	}
	
	function usrIsLoged(){
		return $this->isAuthenticatedApp();
	}
	
	/* FUNCIONES DE AMIGOS */
	
	function frdList(){
		$auth_usr = $this->authenticateApp('gestionar_amigos', $this->parametros['url_friends_script']);
		
		$last_url = Helpers::getSesion()->get('lastUrl');
		
		if($auth_usr === true) {
			
			try {
			
				$req = new Google_Http_Request("https://www.google.com/m8/feeds/contacts/default/full?max-results=1000");
				$val = $this->goo_client_obj->getAuth()->authenticatedRequest($req);
				$xml=  new SimpleXMLElement($val->getResponseBody());
				
				$xml->registerXPathNamespace('gd', 'http://schemas.google.com/g/2005');
				$result = $xml->xpath('//gd:email');
				
			} catch (Exception $e){
				App::setMsg(sprintf(__("Se ha producido un error: %s"), $e->getMessage()), 'error');
				if ($this->parametros['amigos_boton_accion'] == 'popup'){
					echo '<script>opener.location.reload(true);window.close();</script>';
				} else {
					Helpers::redirect($last_url);
				}
				exit;
			}

			$amigos_en_mideas = array();
			
			if(is_array($result) && sizeof($result) > 0){
				
				foreach ($result as $fr){
					# We have an active session, let's check if we have already registered the user
					$existe_user_fb = new Usuario();
					
					$existe_user_fb_c = $existe_user_fb->ListarCount('email="'.(string)$fr->attributes()->address.'" AND estado=1');
					
					if ($existe_user_fb_c == 1){
						$existe_user_fb = $existe_user_fb->Listar('email="'.(string)$fr->attributes()->address.'" AND estado=1');
						if (!in_array($existe_user_fb[0]->id, $amigos_en_mideas) && $existe_user_fb[0]->id != Usuario::getLogedId()){
							$amigos_en_mideas[] = $existe_user_fb[0]->id;
						}
					}
				}
				
				Helpers::getSesion()->set('usuario.facebook_friends', $amigos_en_mideas);
				
				return true;
			} else {
				# For testing purposes, if there was an error, let's kill the script
				App::setMsg(__("Se ha producido un error inesperado"), 'error');
				if ($this->parametros['amigos_boton_accion'] == 'popup'){
					echo '<script>opener.location.reload(true);window.close();</script>';
				} else {
					Helpers::redirect($last_url);
				}
				exit;
			}
		
			if (!$usuario_loged){
				App::setMsg(__("No se ha encontrado el usuario. Error crítico!"));
				if ($this->parametros['amigos_boton_accion'] == 'popup'){
					echo '<script>opener.location.reload(true);window.close();</script>'; 
				} else {
					Helpers::redirect($last_url);
				}
				exit;
			}
			
			$AppUsuario = $user['username'];
			
			Helpers::getSesion()->set('usuario.AppIdUsuario', $id_usuario);
			Helpers::getSesion()->set('usuario.AppUsuario', $AppUsuario);
			Helpers::getSesion()->set('usuario.nivel_acceso', $usuario_loged->nivel_acceso);
			Helpers::getSesion()->set('usuario.google', true);
			
			if ($save_in_cookie)
				self::setLoginCookie($id_usuario);
			
			return true;
			
			// The access token may have been updated lazily.
			Helpers::getSesion()->set('apps.'.$this->nombre.'.token', $client->getAccessToken());
		} else {
			App::setMsg($auth_usr, 'error');
			if ($this->parametros['amigos_boton_accion'] == 'popup'){
				echo '<script>opener.location.reload(true);window.close();</script>';
			} else {
				Helpers::redirect($last_url);
			}
			exit;
		}
		
		return false;
	}
	
	/* FUNCIONES DE ANALYTICS */
	
	function statsVisitas($fecha_inicio, $fecha_fin){
		$defaults = array(
			'fecha_inicio' => date('Y-m-d', strtotime('-1 month')),
			'fecha_fin'   => date('Y-m-d'),
		);
		
		$last_url = Helpers::getSesion()->get('lastUrl');
		
		if (!empty($fecha_inicio)){
			$defaults['fecha_inicio'] = $fecha_inicio;
		}
		if (!empty($fecha_fin)){
			$defaults['fecha_fin'] = $fecha_fin;
		}
		
		$auth_usr = $this->authenticateApp('gestionar_estadisticas', 'api/apps/Google/visits.system-msg', array(/*"https://www.googleapis.com/auth/youtube",*/"https://www.googleapis.com/auth/analytics.readonly","https://www.googleapis.com/auth/userinfo.email", "https://www.googleapis.com/auth/userinfo.profile"));
		
		if ($auth_usr === true){
			
			// y con este cliente creamos el objeto para lanzar consultas a la API
			$service = new Google_Service_Analytics($this->goo_client_obj);
			
			$params_visitas = array(
				'dimensions' => 'ga:date'
			);
			
			try {
				// ahora ya podemos realizar consultas de la forma acostumbrada
				$visits = $service->data_ga->get($this->parametros['gaIdCuenta'], $defaults['fecha_inicio'], $defaults['fecha_fin'], 'ga:visits', $params_visitas);
			} catch (Exception $e) {
				return $e->getMessage();
			}
			
			if (is_array($visits['rows']) && sizeof($visits['rows']) > 0){
				foreach ($visits['rows'] as $row){
					
					$fecha = strtotime(substr($row[0], 0, 4) . '-' . substr($row[0], 4, 2) . '-' . substr($row[0], 6)) * 1000;
					
					$flot_datas_visits[] = '['.$fecha.','.$row[1].']';
				}
			}
			$flot_data_visits = '['.implode(',',$flot_datas_visits).']';
			
			$visitas_totales = $visits['totalsForAllResults']['ga:visits'];
			
			$visitas_array = $visits['rows'];
			
			$info = (array) $visits['profileInfo'];
			
			$info['url'] = 'https://www.google.com/analytics/web/?hl=es&pli=1#report/visitors-overview/a'.$info['accountId'].'w'.$info['internalWebPropertyId'].'p'.$info['profileId'].'/';
			
			$return = array("fecha_inicio"=>$defaults['fecha_inicio'], "fecha_fin"=>$defaults['fecha_fin'], "visitas_totales"=>$visitas_totales, "visitas"=>$visitas_array, "js"=>$flot_data_visits, "info_cuenta"=>$info);
			
			Helpers::getSesion()->set('apps.'.$this->nombre.'.estadisticas', $return);
			 
			return true;
			
		} else {
			App::setMsg($auth_usr, 'error');
			if ($this->parametros['usuarios_boton_accion'] == 'popup'){
				echo '<script>opener.location.reload(true);window.close();</script>';
			} else {
				Helpers::redirect($last_url);
			}
			exit;
		}
		
		return false;
	}
	
	
	/* FUNCIONES DE YOUTUBE */
	
	private static function getIdUsrFromVidFilename($filename){
		if (empty($filename))
			return false;
			
		$info_filename = pathinfo($filename);
		if (empty($info_filename['filename']))
			return false;
			
		$exp = explode("__", $info_filename['filename']);
		
		if (sizeof($exp) == 2)
			return $exp[1];
		else
			return false;
	}
	
	function vidUpload(){
		$auth_app = $this->authenticateApp('gestionar_videos', '', array("https://www.googleapis.com/auth/youtube"/*,"https://www.googleapis.com/auth/analytics.readonly"*/,"https://www.googleapis.com/auth/userinfo.email", "https://www.googleapis.com/auth/userinfo.profile"));
		
		if ($auth_app === true && !empty($_POST)){
			if ($_POST['token_web'] != Helpers::getFormToken() || empty($_FILES['file']['name']))
				return false;
				
			try {
				$this->goo_youtube_obj = new Google_Service_YouTube($this->goo_client_obj);
				
				$upload_path = CONF_REALPATH_APP . 'tmp/';
					
				if (!is_dir($upload_path))
					Helpers::ftpCrearCarpeta($upload_path);
					
				$filename = md5($_FILES['file']['name'].'_'.time()).'__'.Usuario::getLogedId();
					
				if (move_uploaded_file($_FILES['file']['tmp_name'], $upload_path . $_FILES['file']['name'])){
					$origen = $upload_path . $_FILES['file']['name'];
				} else {
					return false;
				}

				$videoPath = $origen;
				
				$titulo = __('Mi vídeo en Mideas');
				$descripcion = __('Subido a través de la app Google de Mideas');
				$tags = array("mideas");
				$categoria = 22;
				
				if (!empty($_POST['titulo']))
					$titulo = $_POST['titulo'];
					
				if (!empty($_POST['descripcion']))
					$titulo = $_POST['descripcion'];
					
				if (!empty($_POST['tags']))
					$titulo = $_POST['tags'];
					
				if (!empty($_POST['categoria']))
					$categoria = $_POST['categoria'];
	
				// Create a snippet with title, description, tags and category ID
				// Create an asset resource and set its snippet metadata and type.
				// This example sets the video's title, description, keyword tags, and
				// video category.
				$snippet = new Google_Service_YouTube_VideoSnippet();
				$snippet->setTitle($titulo);
				$snippet->setDescription($descripcion);
				$snippet->setTags($tags);
				
				// Numeric video category. See
				// https://developers.google.com/youtube/v3/docs/videoCategories/list 
				$snippet->setCategoryId($categoria);
			
				// Set the video's status to "public". Valid statuses are "public",
				// "private" and "unlisted".
				$status = new Google_Service_YouTube_VideoStatus();
				$status->privacyStatus = "public";
			
				// Associate the snippet and status objects with a new video resource.
				$video = new Google_Service_YouTube_Video();
				$video->setSnippet($snippet);
				$video->setStatus($status);
			
				// Specify the size of each chunk of data, in bytes. Set a higher value for
				// reliable connection as fewer chunks lead to faster uploads. Set a lower
				// value for better recovery on less reliable connections.
				$chunkSizeBytes = 1 * 1024 * 1024;
			
				// Setting the defer flag to true tells the client to return a request which can be called
				// with ->execute(); instead of making the API call immediately.
				$this->goo_client_obj->setDefer(true);
			
				// Create a request for the API's videos.insert method to create and upload the video.
				$insertRequest = $this->goo_youtube_obj->videos->insert("status,snippet", $video);
				
				// Create a MediaFileUpload object for resumable uploads.
				$media = new Google_Http_MediaFileUpload(
					$this->goo_client_obj,
					$insertRequest,
					'video/*',
					null,
					true,
					$chunkSizeBytes
				);
				$media->setFileSize(filesize($videoPath));

				// Read the media file and upload it chunk by chunk.
				$status = false;
				$handle = fopen($videoPath, "rb");
				while (!$status && !feof($handle)) {
				  $chunk = fread($handle, $chunkSizeBytes);
				  $status = $media->nextChunk($chunk);
				}
			
				fclose($handle);
			
				// If you want to make other calls after the file upload, set setDefer back to false
				$this->goo_client_obj->setDefer(false);
				
				Helpers::getSesion()->set('apps.'.$this->nombre.'last_video', $status);
				
			} catch (Google_ServiceException $e) {
				var_dumpre($e.getMessage());
				exit;
				return false;
			}
				
			/*$upload_path = CONF_REALPATH_APP . 'uploads/videos/';
			if (!empty($this->parametros['videos_upload_path']))
				$upload_path = CONF_REALPATH_APP . $this->parametros['videos_upload_path'];
				
			if (!empty($_POST["upload_path"]))
				$upload_path = CONF_REALPATH_APP.$_POST["upload_path"];
				
			if (!is_dir($upload_path))
				Helpers::ftpCrearCarpeta($upload_path);
				
			$filename = md5($_FILES['file']['name'].'_'.time()).'__'.Usuario::getLogedId();
				
			if (move_uploaded_file($_FILES['file']['tmp_name'], $upload_path . $_FILES['file']['name'])){
				$origen = $upload_path . $_FILES['file']['name'];
			} else {
				return false;
			}
			
			$ext = 'mp4';
			if (!empty($this->parametros['videos_tipo_codificacion']))
				$ext = $this->parametros['videos_tipo_codificacion'];
				
			$destino = $upload_path . $filename . '.' .$ext;
			$destino_poster = $upload_path . $filename . '.png';
			
			$upload_url = str_replace(CONF_REALPATH_APP, CONF_URL_APP, $upload_path);
			$relative_path = str_replace(CONF_REALPATH_APP, '', $upload_path); 
			
			if (file_exists($destino)){ 
				return false;	
			} else {
				shell_exec('ffmpeg -i "'.$origen.'" -metadata upload_user_email="'.Usuario::getLoged()->email.'" "'.$destino.'" ');
				shell_exec('ffmpeg -itsoffset 4 -i "'.$origen.'" -vcodec png -vframes 1 -an -f rawvideo -s 1280x720 -y "'.$destino_poster.'" && rm -f -r '.$origen);
			}
			$poster_url = $upload_url . $filename . '.png';
			
			$_SESSION[CLAVE_SECRETA]['apps'][$this->nombre]['last_video'] = array("vidUrl"=>$relative_path . $filename . '.' . $ext, "posterVidUrl"=>$relative_path . $filename . '.png');*/
			
			return true;
		} else {
			return false;
		}
	}
	
	function vidRemove(){
		$auth_app = $this->authenticateApp('gestionar_videos', '', array("https://www.googleapis.com/auth/youtube"/*,"https://www.googleapis.com/auth/analytics.readonly"*/,"https://www.googleapis.com/auth/userinfo.email", "https://www.googleapis.com/auth/userinfo.profile"));

		if ($auth_app === true && !empty($_POST['vidUrl']) && $this->vidIsVideo($_POST['vidUrl'])){
			
			try {
				$this->goo_youtube_obj = new Google_Service_YouTube($this->goo_client_obj);
				
				if (!is_object($this->motor_video))
					return __('No hemos encontrado ningún vídeo de YouTube');
				
				if ($videos_yt = $this->goo_youtube_obj->videos->delete($this->motor_video->getVideoId())){
					return $videos_yt;
				} else {
					return true;
				}
			} catch (Google_ServiceException $e) {
				return $e.getMessage();
			}
		} else {
			return __('Se ha producido un error');
		}
	}
	
	function vidModify(){
		return false;
	}
	
	function vidInfo($vidUrl=''){
		$auth_app = $this->authenticateApp('gestionar_videos', '', array("https://www.googleapis.com/auth/youtube"/*,"https://www.googleapis.com/auth/analytics.readonly"*/,"https://www.googleapis.com/auth/userinfo.email", "https://www.googleapis.com/auth/userinfo.profile"));
		
		if (empty($vidUrl))
			return __('Es necesario proporcionar una url relativa del vídeo');
		
		if ($auth_app === true && !empty($_GET['vidUrl'])){
			$realpath_video = CONF_REALPATH_APP . $vidUrl;
			
			if (!is_readable($realpath_video))
				return __('El archivo de vídeo no es legible');
				
			if (!class_exists('getID3'))
				include CONF_REALPATH_APP . 'includes/library/getID3/getid3.php';
				
			if (!class_exists('getID3'))
				return __('No existe la clase getID3 para leer archivos de vídeo');
				
			$getID3 = new getID3;
			$info_video = $getID3->analyze($realpath_video);

			return $info_video;
		} else {
			return __('No se encuentra el archivo de vídeo o la aplicación no está autenticada');
		}
	}
	
	function printButtonVideoUpload($options){
		$auth = $this->authenticateApp('gestionar_videos');
		
		if (!$auth){
			return false;
		} else {
			$default = array("class"=>"btn btn-primary", "text"=>'<i class="'.$this->parametros['icono'].'"></i> '.sprintf(__("Subir vídeo con %s"), $this->nombre), "nameElement"=>"mideasVideoUpBtn", "idElement"=>"mideasVideoUpBtn", "validate"=>"", "upload_path"=>$this->parametros['videos_upload_path'], "accepted_filetypes"=>"", "file_type"=>"");
			
			if (is_array($options)){
				foreach ($options as $kopt=>$opt){
					if (array_key_exists($kopt, $default)){
						$default[$kopt] = $opt;
					}
				}
			}
			
			if (empty($url))
				$url = $this->getApiUrl('uploadVideo', 'json');
				
			$is_admin = false;
			if (Usuario::isLoged(true) && is_object($GLOBALS['AppAdminTpl'])){
				$is_admin = true;
			}
			
			$path_usr = '';
			if (Usuario::isLoged() && !$is_admin){
				$path_usr = 'usr/' . Usuario::getLogedId() . '/';
			}
			
			if ($is_admin){
				if (!empty($default['upload_path'])){
					$up_path = CONF_DIR_UPLOADS . '/' . $default['upload_path'];
				} else {
					$up_path = CONF_DIR_UPLOADS . '/';
				}
			} else {
				if (!empty($default['upload_path'])){
					$up_path = CONF_DIR_UPLOADS . '/' . $path_usr . $default['upload_path'];
				} else {
					$up_path = CONF_DIR_UPLOADS . '/' . $path_usr;
				}
			}
			
			if (!empty($default["accepted_filetypes"]))
				$accepted_filetypes = $default["accepted_filetypes"];
			else
				$accepted_filetypes = "/^video\/(mp4|mpe?g|quicktime|avi)$/i";
				
			$multi_upload = new Uploader(array(
				'id_campo'=>$default["idElement"],
				'nombre_campo'=>'file',
				'acceptFiletypes'=>$accepted_filetypes,
				'iframeTransport' => true,
				'funcs'=>array('submit'=>'
				function (e, data) {
					var that = $(this);
					
					data.url = "'.$url.'";
					data.formData = {token_web: "'.Helpers::getFormToken().'", upload_path: "'.$up_path.md5(time()).'/"};
					that.fileupload("send", data);
					return false;
				}
				',
					'done'=>'
			function (e, data) {
					try {
						if (data.result.data["id"] != undefined && data.result.data["id"] != ""){
								var random_private = Math.floor((Math.random() * 1000000) + 1);
								var youtube_id = data.result.data["id"];
								var url = "https://www.youtube.com/watch?v="+youtube_id;
								$("#input_hidden_'.$default["idElement"].'").val(url);
								$("#input_hidden_'.$default["idElement"].'").trigger("change");
								bootbox.alert("'.__('¡Vídeo subido correctamente!').'");
								$("#iframe_video'.$default["idElement"].'").html("<video id=\"mideas_video\" class=\"video-mediaelements\" controls=\"controls\" preload=\"auto\" width=\"100%\" height=\"400\"><source src=\""+url+"\"></video><a class=\"btn btn-danger btn-block btn-lg\" id=\"txt_btn_del_'.$default["idElement"].'_"+random_private+"\">'.__('Borrar vídeo').'</a>");
								
								$("#txt_btn_del_'.$default["idElement"].'_"+random_private).live("click", function(){
									if ($("#input_hidden_'.$default["idElement"].'").val() != ""){
										bootbox.confirm("¿Seguro que quieres borrar el vídeo?", function(result) {
										  if (result == true){
											  $.post("'.$this->getApiUrl('removeVideo', 'json').'", { token_web: "'.Helpers::getFormToken().'", vidUrl: $("#input_hidden_'.$default["idElement"].'").val() }, function(data){
												  try {
													
													if (data.response == "ok"){
														$("#input_hidden_'.$default["idElement"].'").val("");
														$("#iframe_video'.$default["idElement"].'").html("");
														
														$("#uploader_'.$default["idElement"].'").show();
														
														if (data.response_text != "" && data.response_text != undefined)
															bootbox.alert(data.response_text);
														else
															bootbox.alert("'.__('Vídeo borrado').'");
													} else {
														bootbox.alert(data);
													}
												  } catch (err){
														bootbox.alert(err.message);
														return false;
													}
											  });
										  }
										}); 
									}
								});

								

								$("#uploader_'.$default["idElement"].'").hide();
						} else {
							bootbox.alert("'.__('Se ha producido un error al intentar subir el vídeo.').'");
						}
	
					} catch (err){
						bootbox.alert(err.message);
					}
	
					if (data.error == undefined){
							$("#progress_'.$default["idElement"].' div.progress-bar").css("width","100%").attr("aria-valuenow", "100");
							$("#progress_'.$default["idElement"].' div.progress-bar").text("100%");
							$("#progress_'.$default["idElement"].'").fadeOut(500);
							return false;
					} else {
						bootbox.alert(file.name+" no se ha subido. Error: "+file.error);
						$("#progress_'.$default["idElement"].' div.progress-bar").css("width","100%").attr("aria-valuenow", "100");
						$("#progress_'.$default["idElement"].' div.progress-bar").text("100%");
						$("#progress_'.$default["idElement"].'").fadeOut(500);
						return false;
					}
				
			}'),
				'buttonclass'=>'btn btn-lg btn-block btn-primary btn-file',
				'buttontext'=>'<i class="fa fa-upload"></i><br><big>'.__('Selecciona archivos para subir').'</big><br><small>'.__('O arrastra y suelta archivos de vídeo').'</small>',
				'fileuploadclass'=>'fileupload fileupload-new btn-block',
				'path_campo'=>$up_path,
				'file_type'=>$default['file_type'],
				'singlefile'=>true));
			
			
			
			$output .= '<div style="position:relative;" id="uploader_'.$default["idElement"].'">'.$multi_upload->render_script().'</div>';

			//output = '<a href="'.$url.'" target="_blank" class="popup '.$default['class'].'">'.$default['text'].'</a>';
			
			return $output;
		}
		
	}
	
}
?>